#ifndef _2D_PHYSICS_ENVIRONMENT_H_
#define _2D_PHYSICS_ENVIRONMENT_H_

const float PI = 3.14159265;

#include <math.h>
#include <cmath>

#include <maths\vector2.h>
#include <list>

using std::list;

#include "rigidbody2D.h"
#include "colliders.h"
#include "2d_physics_object.h"
#include "utility_maths.h"

const float GRAVITATIONAL_CONSTANT = -9.8f;

class PhysicsEnvironment2D
{
public:
	PhysicsEnvironment2D();
	PhysicsEnvironment2D(const PhysicsEnvironment2D&);
	~PhysicsEnvironment2D();

	bool Initialize(abfw::Vector2);
	void Shutdown();

	void StepSimulation(float);

	void AddPhysicsObject(PhysicsObject2D*);

private:
	void CheckCollision(PhysicsObject2D&, PhysicsObject2D&);
	bool BoxToBoxCollision(PhysicsObject2D&, PhysicsObject2D&);
	bool CircleToCircleCollision(PhysicsObject2D&, PhysicsObject2D&);
	bool BoxToCircleCollision(PhysicsObject2D&, PhysicsObject2D&);

private:
	list<PhysicsObject2D*> _objectList;
	abfw::Vector2 _gravity;
};

#endif