#ifndef _INPUTCLASS_H_
#define _INPUTCLASS_H_

#include <tgmath.h>
#include <iostream>

const float I_PI = 3.14159265;

#include "abertay_framework.h"
#include <input/sony_controller_input_manager.h>
#include <input/touch_input_manager.h>
#include <system/platform.h>

class InputClass
{
public:
	InputClass();

	bool Initialize(abfw::Platform&);
	bool Frame();
	void Shutdown();

	bool IsButtonPressed(UInt32 button);
	bool AreButtonsPressed(UInt32 buttons);
	bool IsAnyButtonPressed();

	void LeftStickMagnitude(float& x, float& y);
	void RightStickMagnitude(float& x, float& y);
	void ReturnLeftStickData(float& angle, float& intensity);
	void ReturnRightStickData(float& angle, float& intensity);

	bool IsTouchDown(int panel_num, int touch_num);
	void GetTouchPosition(int panel_num, int touch_num, float& posX, float& posY);
	void GetNewTouchPosition(int panel_num, int touch_num, float& moveX, float& moveY);
	bool CalculateSwipe(int panel_num, int touch_num, float& swipeX, float& swipeY);

private:
	float CalculateAngle(float x, float y);
	float CalculateIntensity(float x, float y);

private:
	abfw::SonyControllerInputManager* controller_manager;
	const abfw::SonyController* controller;
	abfw::TouchInputManager* touch_manager_;
	abfw::Touch previousTouchData[2][6];
	float leftXAxisVal, leftYAxisVal, rightXAxisVal, rightYAxisVal;
	float leftStickAngle, rightStickAngle;
	float leftStickIntensity, rightStickIntensity;
};

#endif