#ifndef _RIGIDBODY_H_
#define _RIGIDBODY_H_

#include <maths\vector2.h>
#include <list>

using std::list;

#include "forces.h"

class Rigidbody2D
{
public:
	Rigidbody2D();
	Rigidbody2D(const Rigidbody2D&);
	~Rigidbody2D();

	void Initialize(float mass);

	inline void SetMass(float mass)
	{
		_mass = mass;
	}
	
	inline float GetMass()
	{
		return _mass;
	}

	inline void SetVelocity(abfw::Vector2 velocity)
	{
		_velocity = velocity;
	}

	inline abfw::Vector2 GetVelocity()
	{
		return _velocity;
	}

	inline void ApplyForce(DirectForce force)
	{
		_actingForces.push_back(force);
	}

	inline void PopFrontForce()
	{
		_actingForces.pop_front();
	}

	inline int GetNumForces()
	{
		return _actingForces.size();
	}

	inline DirectForce ReturnForce()
	{
		DirectForce returnForce = _actingForces.front();
		_actingForces.pop_front();
		return returnForce;
	}

	inline void SetAcceleration(abfw::Vector2 accel)
	{
		_acceleration = accel;
	}

	inline abfw::Vector2 GetAcceleration()
	{
		return _acceleration;
	}

private:
	float _mass;
	float _bouncing; //value between 0.0f and 1.0f - unimplemented
	abfw::Vector2 _velocity;
	abfw::Vector2 _acceleration;
	list<DirectForce> _actingForces;
};
#endif