#include "sceneclass.h"

SceneClass::SceneClass()
{
	input_ = NULL;
	platform_ = NULL;
	player_ptr_ = NULL;
}

SceneClass::SceneClass(const SceneClass& other)
{
}

SceneClass::~SceneClass()
{
}

bool SceneClass::Initialize(Platform& platform, InputClass* input, const char* filename)
{
	input_ = input;
	platform_ = &platform;
	end_of_level_ = false;
	timer_ = 0.0f;

	phys_environment.Initialize(abfw::Vector2(0, 20));

	LoadSceneFile(filename);

	return true;
}

void SceneClass::Shutdown()
{
	while (game_objects.size() > 0)
	{
		game_objects.back()->Shutdown();
		delete game_objects.back();
		game_objects.pop_back();
	}

	phys_environment.Shutdown();
}

int SceneClass::Frame(float dt)
{
	phys_environment.StepSimulation(dt);

	if (!end_of_level_)
	{
		timer_ += dt;

		//Reset if the player touches a 'death zone'
		if (player_ptr_->GetPhysObjPtr()->CheckCollisionWith("zone"))
		{
			ResetLevel();
		}

		//End the level if the player touches the end target
		if (player_ptr_->GetPhysObjPtr()->CheckCollisionWith("target"))
		{
			end_of_level_ = true;
		}

		//Reset if the player goes off the screen
		if (player_ptr_->GetPhysObjPtr()->GetPosition().x > platform_->width() || player_ptr_->GetPhysObjPtr()->GetPosition().x <0)
		{
			ResetLevel();
		}
		else if (player_ptr_->GetPhysObjPtr()->GetPosition().y > platform_->height() || player_ptr_->GetPhysObjPtr()->GetPosition().y < 0)
		{
			ResetLevel();
		}

		//Player Controls
		if (input_->IsButtonPressed((ABFW_SONY_CTRL_CROSS)))
		{
			if (player_ptr_->GetPhysObjPtr()->GetRigidbody()->GetVelocity().y == 0)
			{
				player_ptr_->GetPhysObjPtr()->GetRigidbody()->ApplyForce(DirectForce(0, 7500));
			}
			else if (player_ptr_->GetPhysObjPtr()->CheckCollisionWith("wall", collision_position::RIGHT))
			{
				player_ptr_->GetPhysObjPtr()->GetRigidbody()->ApplyForce(DirectForce(15000, 7500));
			}
			else if (player_ptr_->GetPhysObjPtr()->CheckCollisionWith("wall", collision_position::LEFT))
			{
				player_ptr_->GetPhysObjPtr()->GetRigidbody()->ApplyForce(DirectForce(-15000, 7500));
			}
		}

		if (input_->IsButtonPressed((ABFW_SONY_CTRL_RIGHT)))
		{
			if (player_ptr_->GetPhysObjPtr()->GetRigidbody()->GetVelocity().y == 0)
			{
				player_ptr_->GetPhysObjPtr()->GetRigidbody()->ApplyForce(DirectForce(1000, 0));
			}
			else if (player_ptr_->GetPhysObjPtr()->GetRigidbody()->GetVelocity().x <= 0)
			{
				player_ptr_->GetPhysObjPtr()->GetRigidbody()->ApplyForce(DirectForce(2000, 0));
			}
		}

		if (input_->IsButtonPressed((ABFW_SONY_CTRL_LEFT)))
		{
			if (player_ptr_->GetPhysObjPtr()->GetRigidbody()->GetVelocity().y == 0)
			{
				player_ptr_->GetPhysObjPtr()->GetRigidbody()->ApplyForce(DirectForce(-1000, 0));
			}
			else if (player_ptr_->GetPhysObjPtr()->GetRigidbody()->GetVelocity().x >= 0)
			{
				player_ptr_->GetPhysObjPtr()->GetRigidbody()->ApplyForce(DirectForce(-2000, 0));
			}
		}

		if (input_->IsButtonPressed(ABFW_SONY_CTRL_DOWN))
		{
			player_ptr_->GetPhysObjPtr()->GetRigidbody()->ApplyForce(DirectForce(0, -1000));
		}

		if (input_->IsButtonPressed(ABFW_SONY_CTRL_START))
		{
			return 1;
		}

		for (list<GameObject*>::iterator object = game_objects.begin(); object != game_objects.end(); object++)
		{
			(*object)->Frame();
		}

	}
	else
	{
		if (input_->IsButtonPressed(ABFW_SONY_CTRL_START))
		{
			return 1;
		}
	}
	return false;
}

void SceneClass::Render(abfw::SpriteRendererVita* renderer_, abfw::Font& font)
{
	for (list<GameObject*>::iterator object = game_objects.begin(); object != game_objects.end(); object++)
	{
		renderer_->DrawSprite((*object)->GetSprite());
	}

	char time[20];
	std::snprintf(time, 200, "%.1fs", timer_);

	font.RenderText(renderer_, abfw::Vector3(10.0f, 5.0f, -0.9f), 2.0f, 0xff000000, abfw::TJ_LEFT, time);

	if (end_of_level_)
	{
		font.RenderText(renderer_, abfw::Vector3((platform_->width() / 2), (platform_->height() / 8), -0.9f), 8.0f, 0xff000000, abfw::TJ_CENTRE, time);
		font.RenderText(renderer_, abfw::Vector3((platform_->width() / 2), (platform_->height() / 8) + 300, -0.9f), 3.0f, 0xff000000, abfw::TJ_CENTRE, "Press Start to Continue");
	}
}

void SceneClass::LoadSceneFile(const char* filename)
{
	int num_objects;
	int hundreds, tens, ones;
	int bufferPosition = 0;
	char input;
	int values[6];

	File* level_file = platform_->CreateFile();
	GameObject* objectPtr;

	bool success = level_file->Open(filename);
	if (success)
	{
		char* buffer = NULL;
		Int32 file_size;
		success = level_file->GetSize(file_size);
		if (success)
		{
			buffer = new char[file_size];
			Int32 bytes_read;
			success = level_file->Read(buffer, file_size, bytes_read);

			if (success)
				level_file->Close();

			if (success)
			{
				//Read in the object count
				hundreds = buffer[bufferPosition] - '0';
				bufferPosition++;
				tens = buffer[bufferPosition] - '0';
				bufferPosition++;
				ones = buffer[bufferPosition] - '0';
				bufferPosition++;

				num_objects = (hundreds * 100) + (tens * 10) + ones;

				input = buffer[bufferPosition];
				while (input < 48)
				{
					bufferPosition++;
					input = buffer[bufferPosition];
				}

				for (int i = 0; i < num_objects; i++)
				{
					//Read in the object values
					for (int n = 0; n < 5; n++)
					{
						hundreds = buffer[bufferPosition] - '0';
						bufferPosition++;

						tens = buffer[bufferPosition] - '0';
						bufferPosition++;

						ones = buffer[bufferPosition] - '0';
						bufferPosition+=2;

						values[n] = (hundreds * 100) + (tens * 10) + ones;
					}

					values[5] = buffer[bufferPosition] - '0';
					bufferPosition++;

					//Create the game object using the inputted values
					objectPtr = NULL;

					objectPtr = new GameObject;

					switch (values[5])
					{
					case(0):
						objectPtr->Initialize(values[0], values[1], values[3], values[4], "wall", 0xffffffff, platform_->height());
						objectPtr->GetPhysObjPtr()->GetRigidbody()->SetMass(0);
						break;
					case(1):
						objectPtr->Initialize(values[0], values[1], values[3], values[4], "zone", 0xff0000ff, platform_->height());
						objectPtr->GetPhysObjPtr()->GetRigidbody()->SetMass(0);
						break;
					case(2):
						objectPtr->Initialize(values[0], values[1], values[3], values[4], "player", 0xffff0000, platform_->height());
						objectPtr->GetPhysObjPtr()->GetRigidbody()->SetMass(1);
						//Store a separate pointer to the player object for easy manipulation
						player_ptr_ = objectPtr;
						start_position_ = abfw::Vector2(values[0], values[1]);
						break;
					case(3):
						objectPtr->Initialize(values[0], values[1], values[3], values[4], "target", 0xff00ff00, platform_->height());
						objectPtr->GetPhysObjPtr()->GetRigidbody()->SetMass(0);
						break;
					}
					game_objects.push_back(objectPtr);
					phys_environment.AddPhysicsObject(objectPtr->GetPhysObjPtr());
					
					input = buffer[bufferPosition];
					while (input < 48)
					{
						bufferPosition++;
						input = buffer[bufferPosition];
					}
				}
			}
		}
	}
}

void SceneClass::ResetLevel()
{
	timer_ = 0.0f;
	player_ptr_->GetPhysObjPtr()->SetPosition(start_position_);
	player_ptr_->GetPhysObjPtr()->GetRigidbody()->SetVelocity(abfw::Vector2(0.0f, 0.0f));
}