#ifndef _SPRITE_APPLICATION_H
#define _SPRITE_APPLICATION_H

#include <vector>

#include <system/application.h>
#include <graphics/sprite.h>
#include <graphics/sprite_renderer.h>
#include <graphics/font.h>
#include <input/sony_controller_input_manager.h>
#include <assets\png_loader.h>
#include <graphics\texture.h>
#include <graphics\image_data.h>

#include "inputclass.h"
#include "sceneclass.h"
#include "scene_creator_class.h"
#include "scene_select_class.h"
#include "title_scene_class.h"

namespace abfw
{
	class Platform;
}

class SpriteApplication : public abfw::Application
{
private:
	enum class mode{ SCENE_CREATION, SCENE_SELECTION, SCENE, TITLE_SCENE };
public:
	SpriteApplication(abfw::Platform& platform);
	~SpriteApplication();
	void Init();
	void CleanUp();
	bool Update(float ticks);
	void Render();

private:
	bool LoadLevelList();

private:
	InputClass* input_;
	abfw::Font font_;
	abfw::SpriteRenderer* sprite_renderer_;
	SceneClass* scene_;
	SceneCreatorClass scene_creator_;
	SceneSelectClass scene_selector_;
	TitleSceneClass title_scene_;
	mode game_mode_;
	std::vector<string> filenames_;
	int level_count_;

	float frame_rate_;
};

#endif // _SPRITE_APPLICATION_H